<?php

namespace Memsource\MemsourceConnector\Controller;

use Memsource\MemsourceConnector\Domain\Model\ConfigRepository;
use Memsource\MemsourceConnector\Domain\Service\Logger;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;

class ConfigController extends ActionController
{
    /**
     * @var ConfigRepository
     */
    private $configRepository;

    /**
     * @var Logger
     */
    private $logger;

    /**
     * @param ConfigRepository $configRepository
     */
    public function injectConfigRepository(ConfigRepository $configRepository)
    {
        $this->configRepository = $configRepository;
    }

    /**
     * @param Logger $logger
     */
    public function injectLogger(Logger $logger)
    {
        $this->logger = $logger;
    }

    public function defaultAction()
    {
        $config = $this->configRepository->getConfig();
        $logsSize = $this->logger->getLogFileSize();

        $this->view->assign('token', $config->getToken());
        $this->view->assign('debug', $config->isDebug());
        $this->view->assign('logsSize', $logsSize);

        if ($logsSize === 0 && $config->isDebug()) {
            $this->logger->logSystemInfo();
        }
    }

    public function generateTokenAction()
    {
        $this->configRepository->regenerateToken();
        $this->redirect('default');
    }

    public function toggleDebugAction()
    {
        if ($this->configRepository->isDebugEnabled()) {
            $this->configRepository->disableDebug();
        } else {
            $this->configRepository->enableDebug();
        }

        $this->redirect('default');
    }

    public function downloadLogsAction()
    {
        $this->logger->forceDownloadLogFile();
    }

    public function sendLogsAction()
    {
        $baseUri = $this->uriBuilder->getRequest()->getBaseUri();
        $this->logger->sendLogFileToEmail($baseUri);
        $this->redirect('default');
    }

    public function clearLogsAction()
    {
        $this->logger->emptyLogFile();
        $this->redirect('default');
    }
}
