<?php

namespace Memsource\MemsourceConnector\Domain\Entity;

class Page implements \JsonSerializable
{
    private $title;

    private $navTitle;

    private $subtitle;

    private $abstract;

    private $teaser;

    private $bodytext;

    private $alternativeTitle;

    private $keywords;

    private $description;

    private $sysLanguageUid = 0;

    private $l10nParent = 0;

    private $seoTitle;

    private $ogTitle;

    private $ogDescription;

    private $twitterTitle;

    private $twitterDescription;

    private $images;

    /**
     * @param array $data
     */
    public function __construct($data)
    {
        foreach ($data as $key => $value) {
            $property = lcfirst(str_replace('_', '', ucwords($key, '_')));

            if (property_exists(self::class, $property)) {
                $this->$property = $value;
            }
        }
    }

    /**
     * @return string|null
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @return string|null
     */
    public function getNavTitle()
    {
        return $this->navTitle;
    }

    /**
     * @return string|null
     */
    public function getSubtitle()
    {
        return $this->subtitle;
    }

    /**
     * @return string|null
     */
    public function getAbstract()
    {
        return $this->abstract;
    }

    /**
     * @return string|null
     */
    public function getTeaser()
    {
        return $this->teaser;
    }

    /**
     * @return string|null
     */
    public function getBodytext()
    {
        return $this->bodytext;
    }

    /**
     * @return string|null
     */
    public function getAlternativeTitle()
    {
        return $this->alternativeTitle;
    }

    /**
     * @return string|null
     */
    public function getKeywords()
    {
        return $this->keywords;
    }

    /**
     * @return string|null
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @return int
     */
    public function getSysLanguageUid(): int
    {
        return (int) $this->sysLanguageUid;
    }

    /**
     * @return int
     */
    public function getL10nParent(): int
    {
        return (int) $this->l10nParent;
    }

    /**
     * @return bool
     */
    public function isTranslation(): bool
    {
        return $this->getSysLanguageUid() !== 0 && $this->getL10nParent() !== 0;
    }

    /**
     * @return string|null
     */
    public function getSeoTitle()
    {
        return $this->seoTitle;
    }

    /**
     * @return string|null
     */
    public function getOgTitle()
    {
        return $this->ogTitle;
    }

    /**
     * @return string|null
     */
    public function getOgDescription()
    {
        return $this->ogDescription;
    }

    /**
     * @return string|null
     */
    public function getTwitterTitle()
    {
        return $this->twitterTitle;
    }

    /**
     * @return string|null
     */
    public function getTwitterDescription()
    {
        return $this->twitterDescription;
    }

    /**
     * @return Image[]
     */
    public function getImages(): array
    {
        return $this->images ?? [];
    }

    /**
     * @return array
     */
    public function jsonSerialize()
    {
        return get_object_vars($this);
    }
}
