<?php

namespace Memsource\MemsourceConnector\Domain\Model;

use Doctrine\DBAL\Exception\TableNotFoundException;
use Memsource\MemsourceConnector\Exception\NotFoundException;

class CarouselRepository extends BaseRepository
{
    const TABLE_NAME = 'tx_bootstrappackage_carousel_item';

    /**
     * Get carousel item.
     *
     * @param int $contentUid
     * @return array
     */
    public function findByContentAndSourceLang($contentUid)
    {
        try {
            return QueryBuilderFactory::create(self::TABLE_NAME)
                ->select('uid', 'header', 'subheader', 'bodytext')
                ->from(self::TABLE_NAME)
                ->where('tt_content = ' . (int) $contentUid)
                ->andWhere('sys_language_uid = 0')
                ->execute()
                ->fetchAll();
        } catch (TableNotFoundException $e) {
            return [];
        }
    }

    /**
     * Get or create accordion in target lang.
     *
     * @param int $sourcePageId
     * @param int $targetContentId
     * @param int $sourceItemId
     * @param int $targetLangId
     * @return int
     * @throws NotFoundException
     */
    public function getOrCreateTarget($sourcePageId, $targetContentId, $sourceItemId, $targetLangId)
    {
        $queryBuilder = QueryBuilderFactory::create(self::TABLE_NAME);

        $targetId = $this->findIdByLang($sourcePageId, $targetContentId, $sourceItemId, $targetLangId);

        if ($targetId === false) {
            $newItem = $this->findOneById($sourceItemId);

            unset($newItem['uid']);
            $newItem['pid'] = $sourcePageId;
            $newItem['tt_content'] = $targetContentId;
            $newItem['crdate'] = $newItem['tstamp'] = time();
            $newItem['sys_language_uid'] = $targetLangId;
            $newItem['l10n_parent'] = $newItem['t3_origuid'] = $sourceItemId;

            $queryBuilder
                ->insert(self::TABLE_NAME)
                ->values($newItem)
                ->execute();

            $targetId = $this->findIdByLang($sourcePageId, $targetContentId, $sourceItemId, $targetLangId);
        }

        if ($targetId === false) {
            throw new NotFoundException("Unable to create or find carousel_item '$targetContentId' by language '$targetLangId'.");
        }

        return (int) $targetId;
    }

    /**
     * Store translation into accordion.
     *
     * @param int $id
     * @param string $header
     * @param string $subheader
     * @param string $bodytext
     */
    public function update($id, $header, $subheader, $bodytext)
    {
        QueryBuilderFactory::create(self::TABLE_NAME)
            ->update(self::TABLE_NAME)
            ->where("uid = $id")
            ->set('header', $header)
            ->set('subheader', $subheader)
            ->set('bodytext', $bodytext)
            ->execute();
    }
}
