<?php

namespace Memsource\MemsourceConnector\Domain\Model;

use Memsource\MemsourceConnector\Exception\NotFoundException;
use TYPO3\CMS\Core\Site\SiteFinder;

class LanguageRepository
{
    const TABLE_NAME = 'sys_language';

    /**
     * Convert lang code to ID, for example 'en' -> 2.
     *
     * @param string $langCode
     * @param int $sourcePageId
     * @return int
     */
    public function getLangIdByCode($langCode, $sourcePageId): int
    {
        // First, try to find language in site configuration
        try {
            $site = (new SiteFinder)->getSiteByPageId($sourcePageId);

            foreach ($site->getAllLanguages() as $language) {
                if ($language->getTwoLetterIsoCode() === $langCode) {
                    return $language->getLanguageId();
                }
            }
        } catch (\Throwable $e) {
        }

        // If searchnig in site configuration fails, fallback to database config (BW copatibility with older TYPO3)
        $queryBuilder = QueryBuilderFactory::create(self::TABLE_NAME);

        $targetLangId = $queryBuilder
            ->select('uid')
            ->from(self::TABLE_NAME)
            ->where(
                $queryBuilder->expr()->eq(
                    'language_isocode',
                    $queryBuilder->createNamedParameter($langCode)
                )
            )
            ->execute()
            ->fetchColumn();

        if ($targetLangId === false) {
            throw new NotFoundException("Language with code '$langCode' not found.");
        }

        return (int) $targetLangId;
    }
}
