<?php

namespace Memsource\MemsourceConnector\Domain\Service;

use Memsource\MemsourceConnector\Domain\Entity\Image;
use Memsource\MemsourceConnector\Domain\Entity\Page;

class Serializer
{
    /**
     * Serialize page with parts (content) to HTML.
     *
     * @param Page $page
     * @param array $parts
     * @return string
     */
    public function serializePage($page, $parts): string
    {
        // page
        $html = '<div id="memsource-navtitle">' . $page->getNavTitle() . '</div>';
        $html .= '<div id="memsource-subtitle">' . $page->getSubtitle() . '</div>';
        $html .= '<div id="memsource-abstract">' . $page->getAbstract() . '</div>';
        $html .= '<div id="memsource-teaser">' . $page->getTeaser() . '</div>';
        $html .= '<div id="memsource-bodytext">' . $page->getBodytext() . '</div>';
        $html .= '<div id="memsource-alternative-title">' . $page->getAlternativeTitle() . '</div>';
        $html .= '<div id="memsource-keywords">' . $page->getKeywords() . '</div>';
        $html .= '<div id="memsource-description">' . $page->getDescription() . '</div>';
        $html .= '<div id="memsource-seo-title">' . $page->getSeoTitle() . '</div>';
        $html .= '<div id="memsource-twitter-title">' . $page->getTwitterTitle() . '</div>';
        $html .= '<div id="memsource-twitter-description">' . $page->getTwitterDescription() . '</div>';
        $html .= '<div id="memsource-og-title">' . $page->getOgTitle() . '</div>';
        $html .= '<div id="memsource-og-description">' . $page->getOgDescription() . '</div>';
        $html .= '<div id="memsource-content">';
        $html .= $this->serializeImages($page->getImages(), 'page');

        foreach ($parts as $part) {
            // tt_content
            $html .= '<div data-id="' . $part['uid'] . '">';
            $html .= '<div id="memsource-content-header-' . $part['uid'] . '">' . $part['header'] . '</div>';
            $html .= '<div id="memsource-content-subheader-' . $part['uid'] . '">' . $part['subheader'] . '</div>';
            $html .= '<div id="memsource-content-bodytext-' . $part['uid'] . '">' . $part['bodytext'] . '</div>';
            $html .= $this->serializeImages($part['images'], 'content', $part['uid']);

            // carousel
            $html .= '<div id="memsource-content-carousel-items-' . $part['uid'] . '">';
            foreach ($part['carousel_items'] as $item) {
                $html .= '<div data-carousel-item-id="' . $item['uid'] . '">';
                $html .= '<div id="memsource-content-carousel-header-' . $item['uid'] . '">' . $item['header'] . '</div>';
                $html .= '<div id="memsource-content-carousel-subheader-' . $item['uid'] . '">' . $item['subheader'] . '</div>';
                $html .= '<div id="memsource-content-carousel-bodytext-' . $item['uid'] . '">' . $item['bodytext'] . '</div>';
                $html .= $this->serializeImages($item['images'], 'content-carousel', $item['uid']);
                $html .= '</div>';
            }
            $html .= '</div>';

            // accordion
            $html .= '<div id="memsource-content-accordion-items-' . $part['uid'] . '">';
            foreach ($part['accordion_items'] as $item) {
                $html .= '<div data-accordion-item-id="' . $item['uid'] . '">';
                $html .= '<div id="memsource-content-accordion-item-header-' . $item['uid'] . '">' . $item['header'] . '</div>';
                $html .= '<div id="memsource-content-accordion-item-bodytext-' . $item['uid'] . '">' . $item['bodytext'] . '</div>';
                $html .= $this->serializeImages($item['images'], 'content-accordion', $item['uid']);
                $html .= '</div>';
            }
            $html .= '</div>';

            // tab
            $html .= '<div id="memsource-content-tab-items-' . $part['uid'] . '">';
            foreach ($part['tab_items'] as $item) {
                $html .= '<div data-tab-item-id="' . $item['uid'] . '">';
                $html .= '<div id="memsource-content-tab-item-header-' . $item['uid'] . '">' . $item['header'] . '</div>';
                $html .= '<div id="memsource-content-tab-item-bodytext-' . $item['uid'] . '">' . $item['bodytext'] . '</div>';
                $html .= $this->serializeImages($item['images'], 'content-tab', $item['uid']);
                $html .= '</div>';
            }
            $html .= '</div>';

            $html .= '</div>';
        }

        $html .= '</div>';

        return $html;
    }

    /**
     * Serialize images.
     *
     * @param Image[] $images
     * @param string $partName
     * @param int|null $partUid
     * @return string
     */
    private function serializeImages(array $images, string $partName, $partUid = null): string
    {
        if ($partUid === null) {
            $html = '<div id="memsource-' . $partName . '-images">';
        } else {
            $html = '<div id="memsource-' . $partName . '-images-' . $partUid . '">';
        }

        foreach ($images as $image) {
            $html .= '<div data-' . $partName . '-image-id="' . $image->getUid() . '">';
            $html .= '<div id="memsource-' . $partName . '-image-title-' . $image->getUid() . '">' . $image->getTitle() . '</div>';
            $html .= '<div id="memsource-' . $partName . '-image-alternative-' . $image->getUid() . '">' . $image->getAlternative() . '</div>';
            $html .= '<div id="memsource-' . $partName . '-image-description-' . $image->getUid() . '">' . $image->getDescription() . '</div>';
            $html .= '</div>';
        }

        $html .= '</div>';

        return $html;
    }

    /**
     * Deserialize translated content.
     *
     * @param string $serialized
     * @return array
     */
    public function deserializePage($serialized): array
    {
        $dom = $this->createDom($serialized);

        $content = [];

        /** @var \DOMElement $part */
        foreach ($dom->getElementById('memsource-content')->childNodes as $part) {
            if ($part->hasChildNodes() && $part->hasAttribute('data-id')) {
                $id = $part->getAttribute('data-id');

                $content[$id] = [
                    'header' => $this->getInnerHtml($dom->getElementById("memsource-content-header-$id")),
                    'subheader' => $this->getInnerHtml($dom->getElementById("memsource-content-subheader-$id")),
                    'bodytext' => $this->getInnerHtml($dom->getElementById("memsource-content-bodytext-$id")),
                    'images' => [],
                    'carousel' => [],
                    'accordion' => [],
                    'tab' => [],
                ];

                $imagesDom = $dom->getElementById("memsource-content-images-$id");

                if ($imagesDom instanceof \DOMElement && $imagesDom->hasChildNodes()) {
                    foreach ($imagesDom->childNodes as $imageDom) {
                        if ($imageDom instanceof \DOMElement && $imageDom->hasAttribute('data-content-image-id')) {
                            $imageId = $imageDom->getAttribute('data-content-image-id');
                            $content[$id]['images'][$imageId] = [
                                'title' => $this->getInnerHtml($dom->getElementById("memsource-content-image-title-$imageId")),
                                'alternative' => $this->getInnerHtml($dom->getElementById("memsource-content-image-alternative-$imageId")),
                                'description' => $this->getInnerHtml($dom->getElementById("memsource-content-image-description-$imageId"))
                            ];
                        }
                    }
                }

                $carouselItemsDom = $dom->getElementById("memsource-content-carousel-items-$id");
                if ($carouselItemsDom instanceof \DOMElement && $carouselItemsDom->hasChildNodes()) {
                    foreach ($carouselItemsDom->childNodes as $carouselDom) {
                        if ($carouselDom instanceof \DOMElement && $carouselDom->hasAttribute('data-carousel-item-id')) {
                            $carouselId = $carouselDom->getAttribute('data-carousel-item-id');
                            $content[$id]['carousel'][$carouselId] = [
                                'header' => $this->getInnerHtml($dom->getElementById("memsource-content-carousel-header-$carouselId")),
                                'subheader' => $this->getInnerHtml($dom->getElementById("memsource-content-carousel-subheader-$carouselId")),
                                'bodytext' => $this->getInnerHtml($dom->getElementById("memsource-content-carousel-bodytext-$carouselId"))
                            ];

                            $carouselImageDom = $dom->getElementById("memsource-content-carousel-images-$carouselId");

                            foreach ($carouselImageDom->childNodes as $carouselImageChild) {
                                if ($carouselImageChild instanceof \DOMElement && $carouselImageChild->hasAttribute('data-content-carousel-image-id')) {
                                    $imageId = $carouselImageChild->getAttribute('data-content-carousel-image-id');
                                    $content[$id]['carousel'][$carouselId]['images'][$imageId] = [
                                        'title' => $this->getInnerHtml($dom->getElementById("memsource-content-carousel-image-title-$imageId")),
                                        'alternative' => $this->getInnerHtml($dom->getElementById("memsource-content-carousel-image-alternative-$imageId")),
                                        'description' => $this->getInnerHtml($dom->getElementById("memsource-content-carousel-image-description-$imageId"))
                                    ];
                                }
                            }

                        }
                    }
                }

                $accordionItemsDom = $dom->getElementById("memsource-content-accordion-items-$id");
                if ($accordionItemsDom instanceof \DOMElement && $accordionItemsDom->hasChildNodes()) {
                    foreach ($accordionItemsDom->childNodes as $accordionDom) {
                        if ($accordionDom instanceof \DOMElement && $accordionDom->hasAttribute('data-accordion-item-id')) {
                            $accordionId = $accordionDom->getAttribute('data-accordion-item-id');
                            $content[$id]['accordion'][$accordionId] = [
                                'header' => $this->getInnerHtml($dom->getElementById("memsource-content-accordion-item-header-$accordionId")),
                                'bodytext' => $this->getInnerHtml($dom->getElementById("memsource-content-accordion-item-bodytext-$accordionId"))
                            ];

                            $accordionImageDom = $dom->getElementById("memsource-content-accordion-images-$accordionId");

                            foreach ($accordionImageDom->childNodes as $accordionImageChild) {
                                if ($accordionImageChild instanceof \DOMElement && $accordionImageChild->hasAttribute('data-content-accordion-image-id')) {
                                    $imageId = $accordionImageChild->getAttribute('data-content-accordion-image-id');
                                    $content[$id]['accordion'][$accordionId]['images'][$imageId] = [
                                        'title' => $this->getInnerHtml($dom->getElementById("memsource-content-accordion-image-title-$imageId")),
                                        'alternative' => $this->getInnerHtml($dom->getElementById("memsource-content-accordion-image-alternative-$imageId")),
                                        'description' => $this->getInnerHtml($dom->getElementById("memsource-content-accordion-image-description-$imageId"))
                                    ];
                                }
                            }
                        }
                    }
                }

                $tabItemsDom = $dom->getElementById("memsource-content-tab-items-$id");
                if ($tabItemsDom instanceof \DOMElement && $tabItemsDom->hasChildNodes()) {
                    foreach ($tabItemsDom->childNodes as $tabDom) {
                        if ($tabDom instanceof \DOMElement && $tabDom->hasAttribute('data-tab-item-id')) {
                            $tabId = $tabDom->getAttribute('data-tab-item-id');
                            $content[$id]['tab'][$tabId] = [
                                'header' => $this->getInnerHtml($dom->getElementById("memsource-content-tab-item-header-$tabId")),
                                'bodytext' => $this->getInnerHtml($dom->getElementById("memsource-content-tab-item-bodytext-$tabId"))
                            ];

                            $tabImageDom = $dom->getElementById("memsource-content-tab-images-$tabId");

                            foreach ($tabImageDom->childNodes as $tabImageChild) {
                                if ($tabImageChild instanceof \DOMElement && $tabImageChild->hasAttribute('data-content-tab-image-id')) {
                                    $imageId = $tabImageChild->getAttribute('data-content-tab-image-id');
                                    $content[$id]['tab'][$tabId]['images'][$imageId] = [
                                        'title' => $this->getInnerHtml($dom->getElementById("memsource-content-tab-image-title-$imageId")),
                                        'alternative' => $this->getInnerHtml($dom->getElementById("memsource-content-tab-image-alternative-$imageId")),
                                        'description' => $this->getInnerHtml($dom->getElementById("memsource-content-tab-image-description-$imageId"))
                                    ];
                                }
                            }
                        }
                    }
                }
            }
        }

        $images = [];
        $imagesDom = $dom->getElementById('memsource-page-images');

        if ($imagesDom instanceof \DOMElement && $imagesDom->hasChildNodes()) {
            foreach ($imagesDom->childNodes as $imageDom) {
                if ($imageDom instanceof \DOMElement && $imageDom->hasAttribute('data-page-image-id')) {
                    $imageId = $imageDom->getAttribute('data-page-image-id');
                    $images[$imageId] = [
                        'title' => $this->getInnerHtml($dom->getElementById("memsource-page-image-title-$imageId")),
                        'alternative' => $this->getInnerHtml($dom->getElementById("memsource-page-image-alternative-$imageId")),
                        'description' => $this->getInnerHtml($dom->getElementById("memsource-page-image-description-$imageId"))
                    ];
                }
            }
        }

        return [
            'navTitle' => $this->getInnerHtmlById($dom, 'memsource-navtitle'),
            'subtitle' => $this->getInnerHtmlById($dom, 'memsource-subtitle'),
            'abstract' => $this->getInnerHtmlById($dom, 'memsource-abstract'),
            'teaser' => $this->getInnerHtmlById($dom, 'memsource-teaser'),
            'bodytext' => $this->getInnerHtmlById($dom, 'memsource-bodytext'),
            'alternativeTitle' => $this->getInnerHtmlById($dom, 'memsource-alternative-title'),
            'keywords' => $this->getInnerHtmlById($dom, 'memsource-keywords'),
            'description' => $this->getInnerHtmlById($dom, 'memsource-description'),
            'seoTitle' => $this->getInnerHtmlById($dom, 'memsource-seo-title'),
            'twitterTitle' => $this->getInnerHtmlById($dom, 'memsource-twitter-title'),
            'twitterDescription' => $this->getInnerHtmlById($dom, 'memsource-twitter-description'),
            'ogTitle' => $this->getInnerHtmlById($dom, 'memsource-og-title'),
            'ogDescription' => $this->getInnerHtmlById($dom, 'memsource-og-description'),
            'images' => $images,
            'content' => $content
        ];
    }

    /**
     * Get inner HTML of given tag.
     *
     * @param \DOMDocument $dom
     * @param string $id
     * @return string
     */
    private function getInnerHtmlById(\DOMDocument $dom, string $id): string
    {
        return $this->getInnerHtml($dom->getElementById($id));
    }

    /**
     * Create DOM object from given HTML string.
     *
     * @param string $html
     * @return \DOMDocument
     */
    private function createDom($html): \DOMDocument
    {
        $dom = new \DOMDocument();
        $internalErrors = libxml_use_internal_errors(true);
        $html_utf8 = mb_convert_encoding($html, 'HTML-ENTITIES', 'UTF-8');
        $dom->loadHTML($html_utf8);
        libxml_use_internal_errors($internalErrors);

        return $dom;
    }

    /**
     * Get inner HTML string from given DOM object.
     *
     * @param \DOMElement|null $node
     * @return string
     */
    private function getInnerHtml($node): string
    {
        $innerHTML = '';

        if ($node instanceof \DOMElement) {
            foreach ($node->childNodes as $child) {
                $rawInnerHTML = $child->ownerDocument->saveXML($child, LIBXML_NOEMPTYTAG);
                $rawInnerHTML = $this->cleanupEmptyHtmlTags($rawInnerHTML);
                $innerHTML .= $this->removeNewlines($rawInnerHTML);
            }
        }

        return trim($innerHTML);
    }

    /**
     * Cleanup empty HTML tags: '<hr></hr>' -> '<hr />'.
     *
     * @param string $html
     * @return string
     */
    private function cleanupEmptyHtmlTags($html)
    {
        foreach (['hr', 'br'] as $tag) {
            $html = str_replace("<$tag></$tag>", "<$tag />", $html);
        }

        return $html;
    }

    /**
     * Remove "\r".
     *
     * @param string $html
     * @return string
     */
    private function removeNewlines($html)
    {
        return str_replace('&#13;', '', $html);
    }
}
